{% extends "../base.html" %}

{% block content1 %}                                                
<table border="0" cellpadding="0" cellspacing="0" width="100%">
    <tr>
        <td valign="top" class="textContent">

            <table border="0" cellpadding="0" cellspacing="0" width="100%">
                <tr>
                    <td align="center" valign="top" class="status_{{level}}">
                        {{ level.upper() }} [{{alert.name}}]
                    </td>
                </tr>
            </table>
        </td>
    </tr>
</table>
{% end %}

{% block content2 %}
<table border="0" cellpadding="0" cellspacing="0" width="100%">
    <tr>
        <td align="center" valign="top" class="textContent">
            <b>Time:</b> {{ dt.datetime.now().strftime('%H:%M %d/%m/%Y') }}</b> <br/>
            <b>Value:</b> {{ value }}</b> <br/>
        </td>
    </tr>
</table>
{% end %}
